@@ -194,7 +194,7 @@ def test_sampling_simple(gauss_factory):
     assert mu_sampled == pytest.approx(mu_true1, rel=0.07)
     assert sigma_sampled == pytest.approx(sigma_true1, rel=0.07)
 
-@pytest.mark.skip  # bug fixed upsteams TODO(ASAP)
+# @pytest.mark.skip  # bug fixed upsteams TODO(ASAP)
 def test_sampling_multiple_limits():
     gauss_params1 = create_gauss1()
     n_draws = 1000
