@@ -63,7 +63,8 @@ obs1_split = (zfit.Space(obs='obs1', limits=(-2.4, 1.3))
 
 # @pytest.mark.run(order=4)
 @pytest.mark.parametrize("chunksize", [3000, 100000])
-@pytest.mark.parametrize("num_grad", [False, True])
+# skip the numerical gradient due to memory leak bug, TF2.3 fix: https://github.com/tensorflow/tensorflow/issues/35010
+@pytest.mark.parametrize("num_grad", [bo for bo in [False, True] if not bo or zfit.run.get_graph_mode()])
 @pytest.mark.parametrize("spaces", [obs1, obs1_split])
 @pytest.mark.parametrize("minimizer_class_and_kwargs", minimizers)
 @pytest.mark.flaky(reruns=3)
