@@ -3,5 +3,5 @@
 echo "============================ Building package for test ============================"
 python setup.py sdist bdist_wheel  2>&1 | tail -n 15
 twine check dist/*
-rm -r dist  # cleanup
+python setup.py clean --dist --eggs # cleanup
 echo "======================= Finished building package for test ========================"
