@@ -1,5 +1,5 @@
 #  Copyright (c) 2020 zfit
-
+import contextlib
 import itertools
 import warnings
 from collections import OrderedDict
@@ -7,20 +7,20 @@ from typing import Dict, Union, Callable, Optional, Tuple
 
 import colored
 import numpy as np
-import tableformatter as tafo
 from colorama import Style
 from ordered_set import OrderedSet
+from tabulate import tabulate
 
+from .errors import compute_errors
 from .interface import ZfitMinimizer, ZfitResult
 from ..core.interfaces import ZfitLoss, ZfitParameter
+from ..core.parameter import set_values
 from ..settings import run
 from ..util.container import convert_to_container
 from ..util.exception import WeightsNotImplementedError
 from ..util.warnings import ExperimentalFeatureWarning
 from ..util.ztyping import ParamsTypeOpt
 
-from .errors import compute_errors
-
 
 def _minos_minuit(result, params, sigma=1.0):
     fitresult = result
@@ -109,17 +109,17 @@ class FitResult(ZfitResult):
                 won't be altered.
         """
         super().__init__()
+
         self._status = status
         self._converged = converged
         self._params = self._input_convert_params(params)
+        self._params_at_limit = any(param.at_limit for param in self.params)
         self._edm = edm
         self._fmin = fmin
         self._info = info
         self._loss = loss
         self._minimizer = minimizer
         self._valid = True
-        # self.param_error = OrderedDict((p, {}) for p in params)
-        # self.param_hesse = OrderedDict((p, {}) for p in params)
 
     def _input_convert_params(self, params):
         params = ParamHolder((p, {"value": v}) for p, v in params.items())
@@ -177,7 +177,24 @@ class FitResult(ZfitResult):
 
     @property
     def valid(self):
-        return self._valid
+        return self._valid and not self.params_at_limit and self.converged
+
+    @property
+    def params_at_limit(self):
+        return self._params_at_limit
+
+    @contextlib.contextmanager
+    def _input_check_reset_params(self, params):
+        params = self._input_check_params(params=params)
+        old_values = run(params)
+        try:
+            yield params
+        except Exception as error:
+            warnings.warn("Exception occurred, parameter values are not reset and in an arbitrary, last"
+                          " used state. If this happens during normal operation, make sure you reset the values.",
+                          RuntimeWarning)
+            raise
+        set_values(params=params, values=old_values)
 
     def _input_check_params(self, params):
         if params is not None:
@@ -220,11 +237,11 @@ class FitResult(ZfitResult):
         all_params = list(self.params.keys())
         uncached_params = self._get_uncached_params(params=all_params, method_name=error_name)
 
-        if uncached_params:
-            error_dict = self._hesse(params=uncached_params, method=method)
-            self._cache_errors(error_name=error_name, errors=error_dict)
+        with self._input_check_reset_params(params) as params:
+            if uncached_params:  # TODO: this logic doesn't make sense?
+                error_dict = self._hesse(params=uncached_params, method=method)
+                self._cache_errors(error_name=error_name, errors=error_dict)
 
-        params = self._input_check_params(params)
         all_errors = OrderedDict((p, self.params[p][error_name]) for p in params)
         return all_errors
 
@@ -306,20 +323,20 @@ class FitResult(ZfitResult):
         if method == 'zfit_error':
             warnings.warn("'zfit_error' is still experimental and may fails.", ExperimentalFeatureWarning)
 
-        params = self._input_check_params(params)
-        uncached_params = self._get_uncached_params(params=params, method_name=error_name)
+        with self._input_check_reset_params(params) as params:
+            uncached_params = self._get_uncached_params(params=params, method_name=error_name)
 
-        new_result = None
+            new_result = None
 
-        if uncached_params:
-            error_dict, new_result = self._error(params=uncached_params, method=method, sigma=sigma)
-            if new_result is None:
-                self._cache_errors(error_name=error_name, errors=error_dict)
-            else:
-                msg = "Invalid, a new minimum was found."
-                self._cache_errors(error_name=error_name, errors={p: msg for p in params})
-                self._valid = False
-                new_result._cache_errors(error_name=error_name, errors=error_dict)
+            if uncached_params:
+                error_dict, new_result = self._error(params=uncached_params, method=method, sigma=sigma)
+                if new_result is None:
+                    self._cache_errors(error_name=error_name, errors=error_dict)
+                else:
+                    msg = "Invalid, a new minimum was found."
+                    self._cache_errors(error_name=error_name, errors={p: msg for p in params})
+                    self._valid = False
+                    new_result._cache_errors(error_name=error_name, errors=error_dict)
         all_errors = OrderedDict((p, self.params[p][error_name]) for p in params)
 
         return all_errors, new_result
@@ -355,8 +372,8 @@ class FitResult(ZfitResult):
                 method = "minuit_hesse"
             # LEGACY END
 
-        params = self._input_check_params(params)
-        covariance = self._covariance(method=method)
+        with self._input_check_reset_params(params) as params:
+            covariance = self._covariance(method=method)
         covariance = {k: covariance[k] for k in itertools.product(params, params)}
 
         if as_dict:
@@ -375,14 +392,13 @@ class FitResult(ZfitResult):
         return method(result=self, params=params)
 
     def __str__(self):
-        string = Style.BRIGHT + f'FitResult' + Style.NORMAL + f' of\n{self.loss} \nwith\n{self.minimizer}\n'
-        string += tafo.generate_table(
+        string = Style.BRIGHT + f'FitResult' + Style.NORMAL + f' of\n{self.loss} \nwith\n{self.minimizer}\n\n'
+        string += tabulate(
             [[color_on_bool(self.converged), format_value(self.edm, highprec=False),
               format_value(self.fmin)]],
-            ['converged', 'edm', 'min value'],
-            # grid_style=tafo.SparseGrid()
+            ['converged', 'edm', 'min value'], tablefmt='fancy_grid'
         )
-        string += Style.BRIGHT + "Parameters\n"
+        string += '\n\n' + Style.BRIGHT + "Parameters\n" + Style.NORMAL
         string += str(self.params)
         return string
 
@@ -455,7 +471,7 @@ def color_on_bool(value, on_true=colored.bg(10), on_false=colored.bg(9)):
         value_add = on_true
     else:
         value_add = ''
-    value = value_add + str(value)
+    value = value_add + str(value) + colored.bg(15)
     return value
 
 
@@ -477,6 +493,5 @@ class ParamHolder(dict):  # no UserDict, we only want to change the __str__
             rows.append(row)
 
         order_keys = ['name'] + list(order_keys) + ['at limit']
-
-        return tafo.generate_table(rows, order_keys,
-                                   grid_style=tafo.AlternatingRowGrid(colored.bg(15), colored.bg(254)))
+        table = tabulate(rows, order_keys)
+        return table
