@@ -220,7 +220,8 @@ class FunctionWrapperRegistry:
         return concrete_func
 
 
-def function_factory(func=None, **kwargs):
+# equivalent to tf.function
+def function(func=None, **kwargs):
     if callable(func):
         wrapper = FunctionWrapperRegistry()
         return wrapper(func)
@@ -230,9 +231,6 @@ def function_factory(func=None, **kwargs):
         return FunctionWrapperRegistry(**kwargs)
 
 
-tf_function = function_factory
-
-
 # legacy, remove 0.6
 def function_tf_input(*_, **__):
     raise BreakingAPIChangeError("This function has been removed. Use `z.function(wraps='zfit_tensor') or your"
@@ -251,7 +249,7 @@ def py_function(func, inp, Tout, name=None):
     if not settings.options['numerical_grad']:
         warn_advanced_feature("Using py_function without numerical gradients. If the Python code does not contain any"
                               " parametrization by `zfit.Parameter` or similar, this can work out. Otherwise, in case"
-                              " it depends on those, you may want to set `zfit.run.set_mode(autograd=False)`.",
+                              " it depends on those, you may want to set `zfit.run.set_autograd_mode(=False)`.",
                               identifier="py_func_autograd")
 
     return tf.py_function(func=func, inp=inp, Tout=Tout, name=name)
